#!/bin/sh
#
# tc.sh - tc service script - fausto@ngi.it
# <r4f5fajp2oiuhjqyass3nogu55sivxnc42fnwankiynf6o5jpno6exad.onion>

usage() {

    printf "\n"
    printf " ./tc.sh [command]\n\n"
    printf " install ...... compile and install tc\n"
    printf " auth [foo] ... generate authorization keys\n"
    printf "\n"
}

priviledge() {

cd $HOME/.tc/
su << EOF
install -b -m 0755 -o root tc /usr/local/bin/
install -b -m 0755 -o root tc.1 /usr/share/man/man1/
EOF

    if [ "$?" != "0" ]; then
        printf "failed, su(1) problem\n"; exit 1
    fi
}

mkbindir() {

su << EOF
mkdir -p /usr/local/bin/
EOF
}

mkmandir() {

su << EOF
mkdir -p /usr/share/man/man1/
EOF
}

install() {

    printf "installing tc...\n"
    TC=`echo -n $HOME/.tc`
    OS=`uname -s`
    CC=`which cc 2>&-`
    GCC=`which gcc 2>&-`
    FLAGS="-o $TC/tc $TC/tc.c -lpthread -lncurses "
    BUG="-Wall -Wextra -Wstrict-aliasing -Wstrict-prototypes -pedantic -pedantic-errors -Werror -Wunreachable-code -Wcast-align -Wcast-qual -Wctor-dtor-privacy -Wdisabled-optimization -Wformat=2 -Winit-self -Wlong-long -Wmissing-include-dirs -Wold-style-cast -Woverloaded-virtual -Wredundant-decls -Wsign-promo -Wstrict-overflow=5 -Wswitch-default -Wundef -Wno-unused -Wno-variadic-macros -Wno-parentheses -fdiagnostics-show-option -Wuninitialized -Wmissing-declarations -O0 -fno-elide-constructors -Wshadow"

    if [ "$CC" = "" ] && [ "$GCC" = "" ]; then
        printf "C compiler not found\n"; exit 1
    fi

    case "$OS" in

        "DragonFly")
        $CC -o $TC/tc $TC/tc.c -I /usr/include/priv/ncurses \
        -R /lib/priv -L /usr/lib/priv -lprivate_ncurses -lpthread \
        > $TC/tc.log 2>&1
        ;;

        "OpenBSD")
        $CC $FLAGS -Dopenbsd -Wall -Werror -Wstrict-prototypes -pedantic \
        #$CC $FLAGS -Dopenbsd $BUG \
        > $TC/tc.log 2>&1
        ;;

        "FreeBSD")
        $CC $FLAGS > $TC/tc.log 2>&1
        ;;

        "NetBSD")
        $CC -o $TC/tc $TC/tc.c -lpthread -lcurses > $TC/tc.log 2>&1
        ;;

        "Linux")
        sed -i -e "s|string.h|bsd/string.h|g" $TC/tc.c
        $GCC $FLAGS -lbsd > $TC/tc.log 2>&1
        sed -i -e "s|bsd/string.h|string.h|g" $TC/tc.c
        ;;

        *)
        printf "system is $OS, try manually with:\n"
        printf "gcc/cc -o tc tc.c -lpthread -lncurses [-lbsd]\n"; exit 1
        ;;

    esac

    if [ -s "$TC/tc.log" ]; then # not empty = compiling errors
        printf "failed, see tc.log\n"; exit 1

    else

        if [ ! -e "/usr/local/bin" ]; then
            printf "need root priviledge to mkdir /usr/local/bin/\n"
            mkbindir
        fi

        if [ ! -e "/usr/share/man/man1" ]; then
            printf "need root priviledge to mkdir /usr/share/man/man1/\n"
            mkmandir
        fi

        ID=`id -u`

        if [ "$ID" != "0" ]; then
            printf "need root priviledge to install(1)\n"
        fi

        case "$OS" in

            "DragonFly" | "OpenBSD" | "FreeBSD" | "NetBSD")
            priviledge # because "su -c" not permitted for uid != 0
            ;;

            "Linux")
            cd $HOME/.tc/
            su -c 'install -b -m 0755 -o root tc /usr/local/bin;
            install -b -m 0755 -o root tc.1 /usr/share/man/man1/'

            if [ "$?" != "0" ]; then
                printf "failed, su(1) problem\n"; exit 1
            fi
            ;;

            *)
            printf "failed, su(1) or install(1) problem\n"; exit 1
            ;;

        esac

        printf "done, please do 'man tc' now.\n"; exit 0
    fi
}

notice() {

    I="Give the .auth file to clients who need access. If you generate keys not for local onion edit hostname in .auth_private to match the real. Tor must be restarted after each key operation. Each file in .tc/tor/auth/ must contain different hostnames. Revoke client access to your site removing the .auth file from
.tc/tor/hidden_service/authorized_clients/"
    printf "\n$I\n\n"
}

auth() {

    OS=`uname -s`

    if [ "$OS" != "Linux" ]; then
        printf "BSD-like O.S. detected:\n";
        printf " - coreutils are required for gbase32/gbase64\n";
        printf " - LibreSSL 3.7.0 or superior required for x25519 on OpenBSD\n";
    fi

    T="$HOME/.tc/tor"
    H="/hidden_service"
    A="/authorized_clients"

    if [ ! -e "$T" ]; then

        printf "$T not found.\n"; exit 1
    fi

    if [ ! -e "$T$H" ]; then

        printf "$T$H not found.\n"; exit 1
    fi

    if [ ! -e "$T$H$A" ]; then

        mkdir $T$H$A
    fi

    if [ ! -e "$T/auth/" ]; then

        mkdir $T/auth/
    fi

    N=`echo $2 | tr -d "@"`
    D="descriptor:x25519";

    if [ ! -s "$T$H/hostname" ]; then

        printf "$T$H/hostname not found or empty.\n"; exit 1
    fi

    O=`cat $T$H/hostname | cut -c 1-56`

    # ONION SERVICE SIDE

    if [ -e $T$H$A/$N.auth ] || [ -e $T/auth/$N.auth_private ]; then

       printf "key name exist.\n"; exit 1
    fi

    # ed25519 isn't suported by libressl 3.6.0 and precedent
    openssl genpkey -algorithm x25519 -out .tc$N.pem >/dev/null 2>&1

    if [ "$OS" != "Linux" ]; then

        cat .tc$N.pem | grep -v " PRIVATE KEY" | gbase64 -d -w 64\
        | gtail --bytes=32 | gbase32 | sed 's/=//g' > .tc$N.prv

    else

        cat .tc$N.pem | grep -v " PRIVATE KEY" | base64pem -d \
        | tail --bytes=32 | base32 | sed 's/=//g' > .tc$N.prv

    fi

    PRV=`cat .tc$N.prv`

    if [ "$OS" != "Linux" ]; then

        openssl pkey -in .tc$N.pem -pubout | grep -v " PUBLIC KEY" \
        | gbase64 -d -w 64 | gtail --bytes=32 | gbase32 \
        | sed 's/=//g' > .tc$N.pub

    else

        openssl pkey -in .tc$N.pem -pubout | grep -v " PUBLIC KEY" \
        | base64pem -d | tail --bytes=32 | base32 | sed 's/=//g' > .tc$N.pub

    fi

    PUB=`cat .tc$N.pub`

    notice

    printf "$D:$PUB\n" > $T$H$A/$N.auth;

    # CLIENT SIDE
    printf "$O:$D:$PRV\n" > $T/auth/$N.auth_private;
    printf "public key:  $PUB\n" printf "file:  $T$H$A/$N.auth\n\n"
    printf "private key: $PRV\n" printf "file: $T/auth/$N.auth_private\n\n"
    rm .tc$N.p*
}

case "$1" in

    install)
              install
              ;;
    auth)
              if [ -z $2 ]; then

                  usage; exit 1
              fi
              auth $1 $2
              ;;
    *)
              usage
              ;;
esac

exit 0

